"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const index_1 = require("../index");
const test_1 = require("@schematics/angular/utility/test");
function addServerlessAWS(options) {
    const fileName = options.serverless.aws.filename || 'serverless.yml';
    const source = schematics_1.apply(schematics_1.url('../utils/serverless/files/aws'), [
        schematics_1.move(options.directory)
    ]);
    return schematics_1.chain([
        schematics_1.mergeWith(source),
        tree => {
            tree.rename(`${options.directory}/serverless-aws.yml`, `${options.directory}/${fileName}`);
            return tree;
        },
        index_1.addDependencyToPackageJson(options, 'aws-serverless-express', '^3.2.0'),
        index_1.addDependencyToPackageJson(options, 'serverless-apigw-binary', '^0.4.4', true)
    ]);
}
function addServerlessGcloud(options) {
    const fileName = options.serverless.gcloud.filename || 'serverless.yml';
    const source = schematics_1.apply(schematics_1.url('../utils/serverless/files/gcloud'), [
        schematics_1.move(options.directory)
    ]);
    return schematics_1.chain([
        schematics_1.mergeWith(source),
        tree => {
            tree.rename(`${options.directory}/serverless-gcloud.yml`, `${options.directory}/${fileName}`);
            return tree;
        },
        index_1.addDependencyToPackageJson(options, 'firebase-admin', '^5.11.0'),
        index_1.addDependencyToPackageJson(options, 'firebase-functions', '^0.9.1'),
        index_1.addDependencyToPackageJson(options, 'serverless-google-cloudfunctions', '^1.1.1', true)
    ]);
}
function addServerless(options) {
    options.serverless = {
        aws: {},
        gcloud: {}
    };
    const rules = [];
    rules.push(index_1.addOrReplaceScriptInPackageJson(options, "build:deploy", "npm run build:prod && npm run deploy"));
    rules.push(index_1.addOrReplaceScriptInPackageJson(options, "deploy", "serverless deploy"));
    if (options.provider === 'gcloud') {
        rules.push(addServerlessGcloud(options));
    }
    else if (options.provider === 'aws') {
        rules.push(addServerlessAWS(options));
    }
    else {
        options.serverless.aws.filename = 'serverless-aws.yml';
        options.serverless.gcloud.filename = 'serverless-gcloud.yml';
        rules.push(addServerlessAWS(options));
        rules.push(addServerlessGcloud(options));
        rules.push(tree => {
            //add scripts to package.json
            const packageJsonSource = JSON.parse(test_1.getFileContent(tree, `${options.directory}/package.json`));
            delete packageJsonSource.scripts['build:deploy'];
            packageJsonSource.scripts['build:deploy:aws'] = 'npm run build:prod && npm run deploy:aws';
            packageJsonSource.scripts['build:deploy:gcloud'] = 'npm run build:prod && npm run deploy:gcloud';
            packageJsonSource.scripts['deploy:aws'] = 'cp-cli serverless-aws.yml serverless.yml && npm run deploy';
            packageJsonSource.scripts['deploy:gcloud'] = 'cp-cli serverless-gcloud.yml serverless.yml && npm run deploy';
            tree.overwrite(`${options.directory}/package.json`, JSON.stringify(packageJsonSource, null, "  "));
            return tree;
        });
    }
    return schematics_1.chain(rules);
}
exports.addServerless = addServerless;
//# sourceMappingURL=index.js.map